#%% IMPORTS
import matplotlib.pyplot as plt, matplotlib as mpl
import matplotlib.image as mpimg
import matplotlib.cm as cm
import numpy as np, json
from glob import glob
import pandas as pd
import seaborn as sns
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.colors import LinearSegmentedColormap

import figures # defaults for fonts, ticks, etc go here, shared stuff goes in this module
from figures import markersize_s21, multiple_formatter, cmap2d

# plotting helper functions

def plot_img(ax, img) : 
    ax.axis('off')
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)
    ax.imshow(img, aspect = 'equal', interpolation='none') #, cmap = cm.Greys_r)

    
def open_file( f ) :
    ''' open file and return dictionary
    convert keys to floats, and lists to nparrays
    '''
    with open( f ) as json_file:
        data = json.load(json_file)
    data_import = dict()
    for key in data :
        data_import[ float( key ) ] = data[ key ]
        for k2, l in data[key].items() :
            data[ key ][ k2 ] = np.array( l )
    return data_import

def ax_s21( ax ) :
        ''' configure axes for s21 plot
        used in fig. 1 and fig. 2
        '''
        ax.set_ylabel( '$|S_{21}|$ (dB)', labelpad=-13)
        ax.set_xlabel( '$f$ (GHz)', labelpad=-7)
        ax.set_xlim( (4.95, 6.05) )
        ax.set_yticks(( 0, -10, -20, -30 ) )
        ax.set_yticklabels( ( 0, None, None, -30 ) )
        ax.set_xticks( (5, 5.5, 6.0 ) )
        ax.set_xticklabels( ( 5, None, 6 ) )

def plot_s21_temperatures( ax, tplot, cs ) :
    ''' plot some temperature traces as an example

    tplot: temperatures to plot (np array)
    cs: colors for the tplot
    '''

    theory = open_file( 's21_theory.json' )
    exp = open_file( 's21_exp.json' )
    
    for n, t in enumerate( tplot ) :
        c = cs[n]
        dsample = 3
        if n > 0 :
            f = exp[t]['f_hz'][::2*n]/1e9
            mag = exp[t]['mag_db'][::2*n]
        else :
            f = exp[t]['f_hz']/1e9
            mag = exp[t]['mag_db']
        ax.plot( f, mag, color=c, marker='o', 
                markersize=0.7, 
                linestyle='', zorder=3, 
                label= f'$T$={t} K')
        f = theory[t]['f_hz']/1e9
        mag = theory[t]['mag_db']
        ax.plot( f, mag, color=c, linewidth=0.7, zorder=0 )
    ax_s21( ax )

def plot_fvt( ax, tplot, cs ) :
    ''' plot f( T )
    tplot: temperatures that were used for s21 plots
    cs: colors that were used for the s21 plots
    '''
    f = 'f_v_t.json'
    fsp = 'f_v_t_sptheory.json'
    data = json.load( open(f) )
    data_sp = json.load( open(fsp))

    for k, l in data.items() : data[k] = np.array( l )
    for k, l in data_sp.items() : data_sp[k] = np.array( l )
    ax.errorbar( data['t_exp'],  data['f_exp']/1e9, yerr=data['f_err']/1e9, 
        marker='.', linestyle='', color='black', markersize=2,
        zorder=2 )
    for t, c in zip( tplot, cs ) :
        f = data['f_exp'][ data['t_exp'] == t ][0]/1e9

        ax.scatter( t, f, c=[c], s=10, zorder=3, 
            edgecolor='black', linewidth=0.5, label=f'$T$={t} K' )
    ax.legend( fontsize=8, frameon=False )
    ax.plot( data_sp['t_theory'], data_sp['f_stheory']/1e9, 
            linewidth=0.7, color='black' )
    ax.plot( data_sp['t_theory'], data_sp['f_sptheory']/1e9, 
            linewidth=0.7, color='#495A70', linestyle='--' )

    # axis fine-tuning
    ax.set_ylabel( '$f_r$ (GHz)', labelpad=-11,
            horizontalalignment='right', y=0.65)
    ax.set_xlabel( '$T$ (K)', labelpad=-9, x=0.5 )
    ax.set_xlim( (0.05, 1.05) )
    ax.set_yticks( ( 5.3, 5.4, 5.5 ) )
    ax.set_yticklabels( ( 5.3, None, 5.5 ) )
    ax.set_ylim( (5.25, 5.55) )
    ax.set_xticks( np.linspace( 0.2, 1.0, 5 ) )
    ax.set_xticklabels( ( 0.2, None, None, None, 1.0 ) )

def make_bcolors( bplot, ax ) :
    ''' make colors and colorbar for b plot
    bplot: fields to plot

    returns: colors, cbar
    '''
    scale_fac = 1.3

    cmap = sns.light_palette("#0C4186", reverse=True, as_cmap=True)
    
    bnorm = ( bplot - bplot.min() ) / ( bplot.max() - bplot.min() ) / scale_fac
    clist = cmap( np.linspace( 0, 1/scale_fac, 201 ) )
    colors_scaled = LinearSegmentedColormap.from_list( 'blue_scaled', clist )

    norm = mpl.colors.Normalize(vmin=bplot.max(), vmax=bplot.min())

    cbar = None
    
    return colors_scaled( bnorm ), cbar

def open_rbt() :
    ''' open rbt file.
    return dictionary with the data
    '''
    f = 'rbt.json'
    with open( f ) as json_file:
        data = json.load(json_file)
    data_import = dict()
    for key in data :
        data_import[ float( key ) ] = data[ key ]
        for k2, l in data[key].items() :
            data[ key ][ k2 ] = np.array( l )
    return data_import

def plot_bcuts( ax, data, bplot ):
    ''' plot r(t) cuts
    dsample: number of magnetic fields to downsample

    returns: colors
    '''

    colors, cbar = make_bcolors( bplot, ax )
    for b, c in zip( bplot[0:7], colors[0:7] ) :
        cut = data[b]
        ax.plot( cut['t'], cut['r']-13, label=f'$B={b}$ T', 
            color=c, linewidth=0.7 )
    ### fine-tune axes
    ax.set_xlabel( '$T$ (K)', labelpad=-7,
            horizontalalignment='right', x=0.6 )
    ax.set_ylabel( '$\Delta R$ ($\mathrm{\Omega}$)', labelpad=-10 )
    ax.set_yticks( np.linspace( 0, 30, 4 ) )
    ax.set_yticklabels( ( 0, None, None, 30 ) )
    ax.set_xticks( ( 1.0, 1.5 ) )
    ax.set_xticklabels( ( 1.0, 1.5 ) )

    return colors

def plot_tcb( ax, bplot, colors ) :
    ''' plot the fields listed in bplot with the
    colors in colors
    '''
    with open( 'tcb.json' ) as json_file:
        data = json.load( json_file )
    for k, d in data.items():
        data[k] = np.array( d )

    ax.plot( data['b_exp'], data[ 'tcs'], marker='.',
            markersize=4, linestyle='',
            color='black' )

    ax.plot( data['b_theory'], data[ 't_theory'], 
            linewidth=0.7, linestyle='--',
            color='black' )
    ax.set_xlabel( '$B_x$ (T)', labelpad=-7 )
    ax.set_ylim( -0.03, 1.52 )
    ax.set_ylabel( '$T_c$ (K)', labelpad=-15 )
    ax.set_yticks( (0, 0.5, 1.0, 1.5) )
    ax.set_yticklabels( ( 0, None, None, 1.5 ) )
    ax.set_xlim( ( -0.045, 1.05 ) )
    ax.set_xticks( ( 0, 0.5, 1.0 ) )
    ax.set_xticklabels( ( 0, None, 1.0 ) )

# DRAW THE FIGURE

fig = plt.figure(constrained_layout=True, figsize=(7,3.8))
widths = [2.2, 1.0,1.0]
heights = [1, 1]
gs = fig.add_gridspec(ncols=3, nrows =2, width_ratios=widths,
                          height_ratios=heights)

ax1 = fig.add_subplot(gs[:, 0])
ax1.axis('off')
ax3 = fig.add_subplot(gs[0, 1])
ax4 = fig.add_subplot(gs[0, 2])
ax5 = fig.add_subplot(gs[1, 1])
ax6 = fig.add_subplot(gs[1, 2])


# Make the subplots

# S21/temperature plots
tplot = np.array( (0.2, 0.7, 0.8, 0.95, 1.0 ) )
colors = sns.light_palette("#C95011", reverse=True, as_cmap=True)

tnorm = ( tplot - tplot.min() ) / ( tplot.max() - tplot.min() ) / 1.5
cs = colors( tnorm )
plot_s21_temperatures( ax3, tplot, cs )
ax3.set_xlabel( '$f$ (GHz)', labelpad=-9)
plot_fvt( ax4, tplot, cs )

# transport/b plots
data = open_rbt()
bs = list( data.keys() )
bs.sort()
bplot = np.array( ( bs[0:40:6]) )
colors = plot_bcuts( ax5, data, bplot )
plot_tcb( ax6, bplot, colors )

fig.tight_layout( w_pad=0.0, h_pad=1.5, pad=1 )


#### ALIGNED LABELS FOR SUBPLOTS ########
coords = { 'a' : ( 0.00, 0.96 ),
            'b' : ( 0.48, 0.96 ),
            'c' : ( 0.733, 0.96 ),
            'd' : ( 0.48, 0.5 ),
            'e' : ( 0.733, 0.5 ),
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 10 ) 

#%% save the pdf
fig.savefig( "Fig1.pdf" )


