#%% IMPORTS
import matplotlib.pyplot as plt, matplotlib as mpl
import matplotlib.gridspec as gridspec
import numpy as np, json
from pathlib import Path

from figures import figpath

# filled region settings
fill_args = { 'color' : '#C95011', 'alpha' : 0.15, 'linewidth' : 0 }
dsample = 2 # how much to downsample experimental data

def _load_file( f ) :
    ''' load file and convert to nparrays
    returns: dictionary with data in it
    this is used for the angle sweeps
    '''

    with open( figpath / 'fig3' / f ) as json_file :
        data_str = json.load( json_file )
    data = dict()
    for k in data_str : # convert strings to ints
        data[ float( k ) ] = data_str[ k ]

    for sweep in data.values() : # convert to np arrays
        for k, l in sweep.items() :
            sweep[ k ] = np.array( l )
    return data

def get_min_f( data ) :
    ''' return minimum frequency in an angle sweep
    '''
    get_smallest_f = lambda sweep : min( sweep['f'] )
    smallest_f = min( map( get_smallest_f, data.values() ) )
    return smallest_f

from figures import cmap2d

def plot_angle_experiment(ax,  ylim=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    Returns:
        cs, colors used in plot
    '''
    f = 'angle_experiment.json'
    data = _load_file( f )
    colors = cmap2d
    
    bs = np.sort( list( data.keys() ) )
    cs = colors( (bs-bs.min())/bs.max()/1.0 )

    print( f'minimum f:{get_min_f(data)}')
    for i,b in enumerate( bs ) :
        sweeps = data[b]
        c = cs[i]
        ax.errorbar( sweeps['angle']*np.pi/180, 
                sweeps['f']/1e9, 
                yerr= sweeps['f_err' ]/1e9, linestyle='',
                linewidth=0.7, marker='.', 
                elinewidth=0.5,
                markersize=0.5, label=f'$|B|=${b} T', color=c )

    ax.set_yticklabels( [] )
    ax.set_ylim(ylim[0],ylim[1])

    return cs

def plot_angle_theory(ax,  cs, smallest_f=None, ylim=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    cs: colors for plot
    ylim: passed to ax.set_ylim()
    '''
    f = 'angle_theory.json'
    data = _load_file( f )

    for c, b in zip( cs, data.keys() ):
        sweeps = data[b]
        ax.errorbar( sweeps['angle']*np.pi/180, sweeps['f'], 
                linewidth=0.7, label=f'$|B|=${b} T', color=c )

    ax.set_yticklabels( [] )
    ax.set_ylim(ylim[0],ylim[1])
    ax.fill_between(
        np.linspace(np.pi/4, 3*np.pi/4, 100),  # Go from 0 to pi/2
        ylim[0],                          # Fill from radius 0
        ylim[1],                          # To radius 1
        **fill_args
    )

    ax.fill_between(
        np.linspace(-np.pi/4, -3*np.pi/4, 100),  # Go from 0 to pi/2
        ylim[0],                          # Fill from radius 0
        ylim[1],                          # To radius 1
        **fill_args
    )

def load_symasym( file = 'fsymasym.json') :
    """ load fsym/fasym dataset
    Returns:
        data as a dictionary, with entries as nparrays
    """
    with open( figpath / 'fig3' / file ) as json_file :
        data = json.load( json_file )
    for k in data :
        data[k] = np.array( data[k] )
    return data

def plot_fsymasym( ax_sym, ax_asym ) :
    """ make fsym ax_sym, fasym plot on ax_asym
    """
    data = load_symasym()

    frange = ( 5.20, 5.45 ) # y-range for fsym plot

    marker_options = { 'color' : '#234F81',
                'linestyle' : '',
                'markersize' : 1,
                'fmt' : 'o',
                'mfc' : 'None',
                'elinewidth' : 0.5 }
    
    ax_sym.errorbar( data['phi_exp'][::dsample], 
            data['fsym'][::dsample]/1e9, 
            yerr=data['dfsym'][::dsample]/1e9, 
            **marker_options )
    ax_sym.plot( data['phi_theory'], 
            data['fsym_p91_theory'], 
            color='#234F81', linewidth=0.7 )
    ax_sym.hlines( 5.44906155, 0, 360 )
    ax_sym.set_yticks( frange )
    # fill in unstable squares
    ax_sym.set_ylim( 5.15, 5.48 )

    ax_asym.errorbar( data['phi_exp'][::dsample], 
            data['fasym'][::dsample]/1e9, 
            yerr=data['dfsym'][::dsample]/1e9, **marker_options )
    full_range = abs( frange[1] - frange[0] )
    ax_asym.set_ylim( (-0.165, 0.165) )
    ax_asym.set_yticks( (-0.1, 0, 0.1 ) )
    ax_asym.set_yticklabels( (-0.1, None, 0.1) )
    ax_asym.hlines( 0, *ax_asym.get_xlim(), linewidth=1, color='#234F81' )

    fill_lims = ( (45, 180-45), (180+45, 360-45) )
    for a in ( ax_sym, ax_asym ) :
        for xmin, xmax in fill_lims :
            a.axvspan(xmin, xmax, **fill_args)
        a.set_xlabel( '$\\theta$ (deg)', labelpad=-10 )
        a.set_xticks( (0, 360 ) )
    ax_sym.set_ylabel( '$f_{r,\mathrm{sym}}$ (GHz)', labelpad=-2, y=0.49 )
    ax_asym.set_ylabel( '$f_{r,\mathrm{asym}}$ (GHz)', labelpad=-2, y=0.49 )

# DRAW THE FIGURE

fig = plt.figure(figsize=(3.5,3.), constrained_layout=False)
spec = gridspec.GridSpec(ncols=2, nrows=2,
        width_ratios=(1,1),
        height_ratios=(1.4,1),
        figure=fig)

ax = fig.add_subplot( spec[0,0], projection='polar' )
ax2 = fig.add_subplot(spec[0,1], projection='polar' )
ax3 = fig.add_subplot( spec[1,0] )
ax4 = fig.add_subplot( spec[1,1] )

ax.set_xticklabels( (None, None, None, None, None, None ) )
ax2.set_xticklabels( (None, None, None, None, None, None ) )

# Make the subplots
ymin = 4.85 # 4.817989829782215 smallest frequency in theory curves
ylim = (ymin,5.55) # minimum y from theory plot is 4.49 GHz
cs = plot_angle_experiment(ax, ylim=ylim )
ax.tick_params(direction='out', labelsize = 9)
plot_angle_theory(ax2, cs, ylim=ylim )
ax2.tick_params(direction='out',labelsize = 9)
plot_fsymasym( ax3, ax4 )

# subplot labels and legend
coords = { 'a' : ( 0.00, 0.95 ),
            'b' : ( 0.495, 0.95 ),
            'c' : ( 0.00, 0.42 ),
            'd' : ( 0.495, 0.42 )
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 10 )

# Make the legend
handles, labels = ax2.get_legend_handles_labels()
handles = [handles[0], handles[3], handles[1], handles[4], handles[2], handles[5] ]
labels = [labels[0], labels[3], labels[1],labels[4], labels[2], labels[5]]
fig.legend(handles, labels,
    ncol=3, fontsize=8, columnspacing=0.5, 
    frameon=False, handlelength=1,
    handletextpad=0,
    bbox_to_anchor=(0.9, 0.657),
    bbox_transform=fig.transFigure )

fig.tight_layout( pad=1, w_pad=1 )

# how many points are we clipping?
f = 'angle_experiment.json'
data = _load_file( f )

fs = np.array([])
for b in data:
    sweeps = data[b]
    fs = np.append( fs, sweeps['f'] )

nclip = fs[ fs < ymin*1e9 ].size
kept = 1 - nclip/fs.size
print( f"We are clipping {nclip} out of {fs.size} points")
print( f"We are keeping {100*kept:.2f}% of the data points")

#%%
fig.savefig( "Fig3.pdf" )
