##### TEMPERATURE DEPENDENT LOSS #######
#%%
import matplotlib.pyplot as plt, numpy as np, os, json
from pathlib import Path

from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.ticker import FuncFormatter

from figures import cmap2d_r, bstar

# plot function that reads json file
def my_formatter(x, pos):
    """Format 1 as 1, 0 as 0, and all values whose absolute values is between
    0 and 1 without the leading "0." (e.g., 0.7 is formatted as .7 and -0.4 is
    formatted as -.4)."""
    val_str = '{:g}'.format(x)
    if np.abs(x) == 0.0:
        return val_str.replace("0.0", "0", 1)
    else:
        return val_str
major_formatter = FuncFormatter(my_formatter)

def plot_qi_invs(ax=None):
    """Read qi and qi_theory from json files and make a plot of 1/q"""
    with open( f'qi_B0.json' ) as infile:
        qi = json.load( infile )
    for k, d in qi.items() :
        qi[k] = np.array( d )
    if ax is None: 
        fig, ax = plt.subplots()
    inv_qi  = 1./qi['qi']
    inv_qi_err = qi['err'] * inv_qi * inv_qi
    ax.errorbar(qi['T'], inv_qi, yerr=inv_qi_err,
                marker='o',
                markersize=1.5,
                linestyle='', elinewidth = 0.7,
                color='black')

    with open( f'qi_theory.json' ) as infile:
        qi_theory = json.load( infile )
    for k, d in qi_theory.items() :
        qi_theory[k] = np.array( d )

    ax.plot(qi_theory['T'], 1./qi_theory['qi_theory'],
                linestyle='-', linewidth = 0.7,
                color='black')


    return ax

def format_qiplot( ax ):
    """Labelling and formatting the qi plots"""

    ax.xaxis.set_major_formatter(major_formatter)
    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )
    tick0p5 = ax.xaxis.get_major_ticks()[1]
    tick0p5.label.set_position( ( 10, 0 ) )

    ax.set_yscale( 'log' )
    ax.set_xlabel( '$T$ (K)', labelpad=-7, x=0.5 )
    ax.set_xticks( ( 0.1, 0.5, 1 ) )
    ax.set_xticklabels( ['0.1', None, '1'], fontsize=9 )

    ax.set_ylabel( '$Q^{-1}_i$', labelpad=-15, y=0.45 )
    ax.set_ylim([1e-4,1.5e-2])
    ax.set_yticks( ( 1e-4, 1e-3, 1e-2) )
    ax.set_yticklabels( ('$10^{-4}$', None, '$10^{-2}$'), fontsize=9 )

    return ax

def make_cbar( ax, pcm ) :
    ''' make colorbar for diff maps '''
    
    cax = inset_axes(ax,
            width="100%",
            height="3%",  
            loc='upper left',
            bbox_to_anchor=(0, 0, 1, 1.05),
            bbox_transform = ax.transAxes,
            borderpad=0 )
    cbar = ax.figure.colorbar( pcm, ax=ax, cax=cax,
                        orientation='horizontal', ticks=(0, 0.15) )

    cbar.ax.set_ylabel( '$\delta Q_i^{-1}$', size=9, rotation=0, 
                            labelpad=-45, y=1.25 )
    cbar.ax.tick_params( labelsize=9, direction='out', pad=0.0 )
    cbar.set_ticks([0, 0.08])
    cbar.set_ticklabels(['0', '0.08'])
    cbar.ax.xaxis.set_label_position('top')
    cbar.ax.xaxis.set_ticks_position('top')

    return cbar

def plot_cmap( result, b_mesh, t_mesh, vmin=None, vmax=None, ax=None, **kwargs ) :
    ''' plot result, b, t
    return ax, cbar

    '''
    X, Y = b_mesh, t_mesh
    if ax is None:
        fig, ax = plt.subplots(1)
    
    pcm = ax.pcolormesh( Y, X, result, 
            cmap=cmap2d_r, vmin=vmin, vmax=vmax, **kwargs )
    pcm.set_edgecolor('face')

    ax.set_xlabel( '$B_x$ (T)', labelpad=-7, x=0.35 )
    ax.set_xticks( ( 0, 0.5 ) )
    ax.set_xticklabels( ('0.1','0.5'), fontsize=9 )

    ax.set_ylabel( '$T$ (K)', labelpad=-8, y=0.45 )
    ax.set_yticks( (0.1, 0.5, 1.0) )
    ax.set_yticklabels( (0.1, None, 1.0), fontsize=9 )

    ax.xaxis.set_major_formatter(major_formatter)
    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )
    tick0p5 = ax.xaxis.get_major_ticks()[1]
    tick0p5.label.set_position( ( 10, 0 ) )
    return ax, pcm

def plot_qi_inv_bt(ax=None) :
    ''' plot inverse qi( b, t ) as 2d
    '''
    with open( f'qi_inv_bt.json' ) as infile:
        qi_inv = json.load( infile )
    for k, d in qi_inv.items() :
        qi_inv[k] = np.array( d )

    qi_inv['qi_invs'] -= qi_inv['qi_invs'][0,:]

    ax, pcm = plot_cmap( qi_inv['qi_invs'], qi_inv['t_mesh'], 
                            qi_inv['b_mesh'], ax=ax)

    return ax, pcm

# Plotting
fig, axes = plt.subplots(nrows=1, ncols=2, figsize=[3.5,1.8])

ax1 = plot_qi_invs(axes[0])
ax1 = format_qiplot(ax1)

ax2, pcm = plot_qi_inv_bt(axes[1])

bstar(ax2,  dir='out', padd=2, size =9)

make_cbar( ax2, pcm)
fig.subplots_adjust(top=0.85,
                    bottom=0.14,
                    left=0.115,
                    right=0.96,
                    wspace=0.3
)

ax2.text(0.015, 0.12, 'EXP.', 
    horizontalalignment='left',
    color='white',
    fontsize=9,
    verticalalignment='top', transform=ax2.transAxes)

coords = { 'a' : ( 0.00, 0.92 ),
            'b' : ( 0.49, 0.92 ),
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 10 ) 

#%%
fig.savefig(f'Fig4.pdf' )

# %%
