#%% PLOT 3 ANGLE SWEEPS AND MISALIGNMENT S58
import numpy as np, json
import matplotlib.pyplot as plt
from pathlib import Path
from figures import figpath

export_dir = figpath / 'supfigs'

fontsize = 10

cblue = "#3C91E6"
cpink = "#F0544F"
cgrey = "#7A7A7A"
cdark = "#041F1E"

def _load_file( f ) :
    ''' load file and convert to nparrays
    returns: dictionary with data in it
    '''
    with open( f ) as json_file :
        data_str = json.load( json_file )

    data = dict()

    for k in data_str : # convert strings to ints
        data[ k ] = data_str[ k ]

    for sweep in data.values() : # convert to np arrays
        for k, l in sweep.items() :
            if isinstance(l, list):
                sweep[ k ] = np.array( l )
    
    return data

def _get_min_f( data ) :
    ''' return minimum frequency in an angle sweep
    '''
    get_smallest_f = lambda sweep : min( sweep['f'] )
    smallest_f = min( map( get_smallest_f, data.values() ) )
    return smallest_f

def plot_angle_experiment( ax = None, smallest_f=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    '''
    f = f'{export_dir}/s58_misalign.json'
    data = _load_file( f )
    if smallest_f is None:
        smallest_f = _get_min_f( data )/1e9

    if ax is None:
        fig = plt.figure()
        ax = fig.add_axes( [0.20, 0.1, 0.9, 0.8],
                    projection='polar' )
    fig = ax.get_figure()

    swp = data['#175']
    ax.errorbar( swp['angle']*np.pi/180, swp['f']/1e9, 
            yerr= swp['f_err' ]/1e9, linestyle='',
            linewidth=0.5, marker='o', color=cblue,
            markersize=1, label='y-Misaligned' )
    
    swp = data['#464']
    ax.errorbar( swp['angle']*np.pi/180, swp['f']/1e9, 
            yerr= swp['f_err' ]/1e9, linestyle='',
            linewidth=0.5, marker='o', color=cpink,
            markersize=1, label='xy-Misaligned' )

    swp = data['#477']
    ax.errorbar( swp['angle']*np.pi/180, swp['f']/1e9, 
            yerr= swp['f_err' ]/1e9, linestyle='',
            linewidth=0.5, marker='o', color=cdark,
            markersize=1, label='Optimal' )

    fontdict = {'fontsize': 10,
                'fontweight': 1,
                'verticalalignment': 'center',
                'horizontalalignment': 'center'}
    ax.set_rlim( smallest_f )
    ax.set_yticklabels( [] )
    ax.set_xticks( np.pi/180. * np.linspace(0,360,8,endpoint=False))
    ax.set_xticklabels(['0$\degree$', None, '90$\degree$', None, \
                        '180$\degree$    ', None, '270$\degree$', None],
                        fontdict = fontdict
                        )

    ax.tick_params( pad = 2 )

    ax.legend( fontsize=fontsize, loc='upper left' , 
                handlelength=0.,
                bbox_to_anchor=(0.0,0.23), 
                bbox_transform=fig.transFigure )

    ax.set_position( [0.05, 0.14, 0.6, 0.77])

def plot_comp_z( ax = None ):
    f = f'{export_dir}/s58_misalign.json'
    data = _load_file( f )

    if ax is None:
        fig,ax = plt.subplots()
    
    bz = data["#477"]['bz_comp']

    ax.plot( bz*1e3, data["#477"]['z_compfield']*1e3,
                marker='.', markersize=3, ls='',
             color=cgrey )

    ax.plot( bz*1e3, bz*data["#175"]['z_slope']*1e3,
                ls='--',
                color=cblue )

    ax.plot( bz*1e3, bz*data["#464"]['z_slope']*1e3,
                ls=':',
                color=cpink )
    
    ax.plot( bz*1e3, bz*data["#477"]["z_slope"]*1e3 + data['#477']['z_intercept']*1e3,
                color=cdark, lw=0.7
                )

    ax.set_xticks([0, 800])
    ax.set_xticklabels(['0', '800'], fontsize=fontsize)
    ax.set_xlabel(r"$B_x$ (mT)", fontsize=fontsize,
                    labelpad=-12)
    
    ax.set_yticks([0, -10])
    ax.set_yticklabels(['0', '-10'], fontsize=fontsize)
    ax.set_ylabel(r"$B_{\perp}$ (mT)", fontsize=fontsize,
                    labelpad=-12, y=0.56)
    ax.text( x=20, y=-11, s=f"Misaligned \n x-slope", 
                fontsize=fontsize )

def plot_comp_y( ax = None ):
    f = f'{export_dir}/s58_misalign.json'
    data = _load_file( f )

    if ax is None:
        fig,ax = plt.subplots()
    
    by = data["#477"]['by_comp']

    ax.plot( by*1e3, data["#477"]['y_compfield']*1e3,
                marker='.', markersize=3, ls='',
             color=cgrey )

    ax.plot( by*1e3, by*data["#175"]['y_slope']*1.0e3,
                ls='--',
                color=cblue )

    ax.plot( by*1e3, by*data["#464"]['y_slope']*1e3,
                ls=':',
                color=cpink )
    
    ax.plot( by*1e3, by*data["#477"]["y_slope"]*1e3 + data['#477']['y_intercept']*1e3,
                color=cdark, lw=0.7
                )

    ax.set_xticks([0, 600])
    ax.set_xticklabels(['0', '600'], fontsize=fontsize)
    ax.set_xlabel(r"$B_y$ (mT)", fontsize=fontsize,
                    labelpad=-12)
    
    ax.set_yticks([0, 10])
    ax.set_yticklabels(['0', '10'], fontsize=fontsize)
    ax.set_ylabel(r"$B_{\perp}$ (mT)", fontsize=fontsize,
                    labelpad=-12, y=0.4)
    ax.text( x=20, y=9, s=f"Misaligned \n y-slope", 
                fontsize=fontsize )



fig = plt.figure( constrained_layout=False, figsize=[3.5*1.7,3.5])

gs = fig.add_gridspec( nrows=1, ncols=1, left=0.0, right=0.65 )

axpolar = fig.add_subplot( gs[0,0], projection='polar')

gs1 = fig.add_gridspec( nrows=2, ncols=1, left= 0.70, right=0.98,
                            top=0.96, bottom=0.1,
                            wspace=0, hspace=0.25 )

ax1 = fig.add_subplot(gs1[0,0])
plot_comp_z( ax1 )

ax2 = fig.add_subplot(gs1[1,0])
plot_comp_y(ax2)

plot_angle_experiment( ax=axpolar )

coords = { 'a' : ( 0.02, 0.94 ),
            'b' : ( 0.63, 0.94 ),
            'c' : ( 0.63, 0.5 )
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 11 ) 

fig.savefig(f'{export_dir}/sup58_misalign.pdf', format='pdf', dpi=1200)
# %%

