#%% PLOT POWER SWEEP FROM S58
import matplotlib.pyplot as plt, numpy as np, os, json
from pathlib import Path
import seaborn as sns
import figures 

from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import FuncFormatter
from matplotlib import ticker

export_dir = Path( __file__ ).parents[0]

fontsize_ = 11

def dbm2nphoton(x):
    return 10**(x/10.) / 10**(p1dbm/10.)

def nphoton2dbm(x, p1dbm):
    return 10*np.log10(x+1e-300) + p1dbm

def my_formatter(x, pos):
    """Format 1 as 1, 0 as 0, and all values whose absolute values is between
    0 and 1 without the leading "0." (e.g., 0.7 is formatted as .7 and -0.4 is
    formatted as -.4)."""
    val_str = '{:g}'.format(x)
    if np.abs(x) == 0.0:
        return val_str.replace("0.0", "0", 1)
    else:
        return val_str
major_formatter = FuncFormatter(my_formatter)

def plot_fr(ax=None, fname='powersweep_b0_s58.json', label = " ", color='black'):
    """plot powersweep_*.json"""
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )
    
    if ax is None: 
        fig, ax = plt.subplots()
    
    ax.errorbar(data['power_in_dbm'],
                data['fr_ghz'], yerr=data['err_ghz'],
                marker='o',
                markersize=2.5,
                linestyle='', elinewidth = 0.7,
                color=color, label = label,
                alpha=0.7 )

    xlim=[-132, -78]
    fmean = data['fmean']
    errmean = data['errmean']
    fmeans = np.array([fmean, fmean])

    ax.axhline( fmean, color='red', linewidth=2 )
    ax.fill_between( xlim, fmeans+errmean, fmean-errmean,
                    color='red', alpha=0.15 )

    ax.get_figure().tight_layout()

    return ax

def plot_invqi(ax=None, fname='powersweep_b0_s58.json', label = " ", color='black'):
    """plot powersweep_*.json"""
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )
    
    if ax is None: 
        fig, ax = plt.subplots()
    
    ax.errorbar(data['power_in_dbm'],
                data['inv_qi'], yerr=data['inv_qi_err'],
                marker='o',
                markersize=2.5,
                linestyle='', elinewidth = 0.7,
                color=color, label = label,
                alpha=0.7 )

    xlim=[-132, -78]
    inv_qi_mean = data['inv_qi_mean']
    errmean = data['inv_qi_errmean']
    inv_qi_means = np.array([inv_qi_mean, inv_qi_mean])

    ax.axhline( inv_qi_mean, color='red', linewidth=2 )
    ax.fill_between( xlim, inv_qi_mean+errmean, inv_qi_mean-errmean,
                    color='red', alpha=0.15 )

    ax.get_figure().tight_layout()

    return ax

def format_frplot(ax, fname='powersweep_b0_s58.json'):
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )

    p1dbm = data['p1dbm']

    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )

    # config y axis
    ax.set_ylim([4.190, 4.198])
    ax.set_yticks( [4.190, 4.192, 4.194, 4.196, 4.198])
    ax.set_yticklabels( [4.190, None, None, None, 4.198], fontsize=fontsize_)
    ax.set_ylabel(r"$f_r$ [GHz]", fontsize=fontsize_,
                    labelpad=-25, y=0.5)

    # config x axis
    ax.xaxis.set_major_formatter(major_formatter)
    xlim = [-132, -78]
    ax.set_xlim(xlim)
    ax.set_xticks( [-130, -120, -110, -100, -90, -80] )
    ax.set_xticklabels( [-130, None, None, None, None, -80], fontsize=fontsize_ )
    ax.set_xlabel(r"$P_{in}$ [dBm]", fontsize=fontsize_,
                    labelpad=-10, x=0.5)

    # config x2 axis
    secax = ax.twiny()
    secax.set_xlim( ax.get_xlim() )
    ax2ticks_nphoton = 10**np.array([0.1,1,2,3,4,5])
    ax2ticks = nphoton2dbm( ax2ticks_nphoton, p1dbm )
    secax.set_xticks( ax2ticks )
    secax.set_xticklabels( ['0.1', None, None, None, None, r'$10^5$'], fontsize=fontsize_ )
    secax.set_xlabel(r"$N_{photon}$", fontsize=fontsize_,
                        labelpad=-8, x=0.5)

    return fig, ax

def format_qiplot(ax, fname='powersweep_b0_s58.json'):
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )

    p1dbm = data['p1dbm']

    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )

    # config y axis
    ax.set_yscale('log')
    nulform = ticker.NullFormatter()
    ax.get_yaxis().set_minor_formatter(nulform)

    logform = ticker.ScalarFormatter(useMathText=True)
    logform.set_scientific(True)
    logform.set_powerlimits((-4,0))
    ax.yaxis.set_major_formatter(logform)

    ax.set_ylim([4e-4, 2e-3])
    ax.set_yticks( [4e-4, 2e-3])
    ax.set_yticklabels( ['$4.10^{-4}$', '$2.10^{-3}$'], fontsize=fontsize_)
    ax.set_ylabel(r"$Q_i^{-1}$", fontsize=fontsize_,
                    labelpad=-25, y=0.5)
    
    # config x axis
    ax.xaxis.set_major_formatter(major_formatter)
    xlim=[-132, -78]
    ax.set_xlim(xlim)
    ax.set_xticks( [-130, -120, -110, -100, -90, -80] )
    ax.set_xticklabels( [-130, None, None, None, None, -80], fontsize=fontsize_ )
    ax.set_xlabel(r"$P_{in}$ (dBm)", fontsize=fontsize_,
                        labelpad=-10, x=0.5)

    # config x2 axis
    secax = ax.twiny()
    secax.set_xlim( ax.get_xlim() )
    ax2ticks_nphoton = 10**np.array([0.1,1,2,3,4,5])
    ax2ticks = nphoton2dbm( ax2ticks_nphoton, p1dbm )
    secax.set_xticks( ax2ticks )
    secax.set_xticklabels( ['0.1', None, None, None, None, r'$10^5$'], fontsize=fontsize_ )
    secax.set_xlabel(r"$N_{photon}$", labelpad=-8, x=0.5, fontsize=fontsize_)

    return fig, ax

def format_frplot_mf(ax, fname='powersweep_b350_s58.json'):
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )

    p1dbm = data['p1dbm']

    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )

    # config y axis
    ylim = [3.782, 3.788]
    ax.set_ylim(ylim)
    ax.set_yticks( ylim )
    ax.set_yticklabels( ylim, fontsize=fontsize_ )
    ax.set_ylabel(r"$f_r$ [GHz]", labelpad=-25, y=0.5, fontsize=fontsize_)

    # config x axis
    ax.xaxis.set_major_formatter(major_formatter)
    xlim=[-132, -78]
    ax.set_xlim(xlim)
    ax.set_xticks( [-130, -120, -110, -100, -90, -80] )
    ax.set_xticklabels( [-130, None, None, None, None, -80], fontsize=fontsize_)
    ax.set_xlabel(r"$P_{in}$ [dBm]", labelpad=-10, x=0.5, fontsize=fontsize_)

    # config x2 axis
    secax = ax.twiny()
    secax.set_xlim( ax.get_xlim() )
    ax2ticks_nphoton = 10**np.array([0.1,1,2,3,4,5])
    ax2ticks = nphoton2dbm( ax2ticks_nphoton, p1dbm)
    secax.set_xticks( ax2ticks )
    secax.set_xticklabels( ['0.1', None, None, None, None, r'$10^5$'], fontsize=fontsize_ )
    secax.set_xlabel(r"$N_{photon}$", labelpad=-8, x=0.5, fontsize=fontsize_)

    return fig, ax

def format_qiplot_mid(ax, fname='powersweep_b350_s58.json'):
    with open( f'{export_dir}/'+fname ) as infile:
        data = json.load( infile )

    p1dbm = data['p1dbm']

    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )

    # config y axis
    ax.set_yscale('log')
    nulform = ticker.NullFormatter()
    ax.get_yaxis().set_minor_formatter(nulform)

    logform = ticker.ScalarFormatter(useMathText=True)
    logform.set_scientific(True)
    logform.set_powerlimits((-4,0))
    ax.yaxis.set_major_formatter(logform)

    ylim = [8e-3, 1e-2]
    ax.set_ylim(ylim)
    ax.set_yticks( ylim)
    ax.set_yticklabels( [r'$8\times10^{-3}$', r'$10^{-2}$'], fontsize=fontsize_)
    ax.set_ylabel(r"$Q_i^{-1}$", labelpad=-25, y=0.5, fontsize=fontsize_)
    

    # config x axis
    ax.xaxis.set_major_formatter(major_formatter)
    xlim=[-132, -78]
    ax.set_xlim(xlim)
    ax.set_xticks( [-130, -120, -110, -100, -90, -80] )
    ax.set_xticklabels( [-130, None, None, None, None, -80], fontsize=fontsize_ )
    ax.set_xlabel(r"$P_{in}$ (dBm)", labelpad=-10, x=0.5, fontsize=fontsize_)

    # config x2 axis
    secax = ax.twiny()
    secax.set_xlim( ax.get_xlim() )
    ax2ticks_nphoton = 10**np.array([0.1,1,2,3,4,5])
    ax2ticks = nphoton2dbm( ax2ticks_nphoton, p1dbm )
    secax.set_xticks( ax2ticks )
    secax.set_xticklabels( ['0.1', None, None, None, None, r'$10^5$'], fontsize=fontsize_ )
    secax.set_xlabel(r"$N_{photon}$", labelpad=-8, x=0.5, fontsize=fontsize_)

    return fig, ax
    
fig, ax = plt.subplots(nrows=2, ncols=2, 
                        figsize=[7, 6.4],
                        constrained_layout=True )

plot_fr(ax[0,0], 'powersweep_b0_s58.json')
plot_fr(ax[0,1], 'powersweep_b350_s58.json')
plot_invqi(ax[1,0], 'powersweep_b0_s58.json')
plot_invqi(ax[1,1], 'powersweep_b350_s58.json')

format_frplot(ax[0,0], 'powersweep_b0_s58.json')
format_qiplot(ax[1,0], 'powersweep_b0_s58.json')

format_frplot_mf(ax[0,1], 'powersweep_b350_s58.json')
format_qiplot_mid(ax[1,1], 'powersweep_b350_s58.json')

fig.subplots_adjust(top=0.95,
                    bottom=0.05,
                    left=0.09,
                    right=0.97,
                    hspace=0.3,
                    wspace=0.35
)
coords = { 'a' : ( 0.02, 0.97 ),
            'b' : ( 0.52, 0.97 ),
            'c' : ( 0.02, 0.48 ),
            'd' : ( 0.52, 0.48 )
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = fontsize_+2 ) 

fig.savefig(f'{export_dir}/powersweep_s58.pdf', format='pdf', dpi=1200)
fig.show()
# %%
